# Copyright (c) 2010 gocept gmbh & co. kg
# See also LICENSE.txt

import cookielib
import urllib
import urllib2
import re


MATCH_NEW_TICKET_NO = re.compile(r'Ticket\s*(\d+)')


class Ticket(dict):

    def __init__(self, rt):
        super(Ticket, self).__init__()
        self.rt = rt

    @classmethod
    def fromResponse(cls, response, rt):
        ticket = Ticket(rt)
        response = response[:]
        assert "RT/3.4.3 200 Ok\n" == response.pop(0)
        assert "\n" == response.pop(0)
        assert "\n" == response.pop()
        for line in response:
            key, value = line.split(':', 1)
            ticket[key] = value.strip()
        ticket['id'] = ticket['id'].replace('ticket/', '')
        return ticket

    def edit(self, data):
        self.rt.edit_ticket(self['id'], data)


class RT(object):

    DATE_FORMAT = '%a %b %d %H:%M:%S %Y'

    def __init__(self, user, password, url):
        self.cookies = cookielib.LWPCookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookies))
        self.url = '%s/REST/1.0' % url
        self.user = user
        self.password = password
        self.login()

    def open(self, url, data=None):
        if data is not None:
            data = urllib.urlencode(data)
        return self.opener.open(url, data)

    def login(self):
        response = self.open(self.url, {'user': self.user, 'pass': self.password})
        response.read()

    def show_ticket(self, ticket_id):
        data = {}
        response = self.open(self.url+'/ticket/%s/show' % ticket_id)
        return Ticket.fromResponse(response.readlines(), self)

    def search(self, query):
        query = urllib.quote(query)
        tickets = []
        response = self.open(self.url+'/search/ticket?format=i&query=%s' % query)
        lines = response.readlines()
        assert "RT/3.4.3 200 Ok\n" == lines.pop(0)
        assert "\n" == lines.pop(0)
        for line in lines:
            line = line.strip()
            prefix, ticket = line.split('/')
            assert prefix == 'ticket'
            tickets.append(self.show_ticket(ticket))
        return tickets

    def new_ticket(self, data):
        data['id'] = 'ticket/new'
        if 'Text' in data:
            data['Text'] = data['Text'].replace('\n', '\n  ')
        content = dict(content='\n'.join('%s: %s' % (key, value)
                                         for key, value in data.items()))
        r = self.open(self.url+'/ticket/new', content)
        result = r.read()
        m = MATCH_NEW_TICKET_NO.search(result)
        try:
            ticket = m.groups()[0]
        except KeyError:
            raise RuntimeError(result)
        return self.show_ticket(ticket)

    def edit_ticket(self, ticket_id, data):
        content = dict(content='\n'.join('%s: %s' % (key, value)
                                         for key, value in data.items()))
        r = self.open(self.url+'/ticket/%s/edit' % ticket_id, content)
        r.read()
